@@ -6,6 +6,10 @@
 #ifndef BIBTEXOUT_H
 #define BIBTEXOUT_H
 
+#include <stdio.h>
+#include "fields.h"
+#include "bibutils.h"
+
 /* bibtexout output options */
 #define BIBOUT_FINALCOMMA (2)
 #define BIBOUT_SINGLEDASH (4)
@@ -14,7 +18,9 @@
 #define BIBOUT_UPPERCASE (32)
 #define BIBOUT_STRICTKEY (64)
 
-extern void bibtexout_write( fields *info, FILE *fp, int format_opts, 
+extern void bibtexout_write( fields *info, FILE *fp, param *p, 
 		unsigned long refnum );
+extern void bibtexout_writeheader( FILE *outptr, param *p );
+
 
 #endif
