@@ -16,13 +16,20 @@
 #include "utf8.h"
 #include "modsout.h"
 #include "modstypes.h"
+#include "marc.h"
 
-extern char progname[];
+static int
+increment_level( int level, int amt )
+{
+	if ( level > -1 ) return level+amt;
+	else return level-amt;
+}
 
 static void
 output_tab0( FILE *outptr, int level )
 {
 	int i;
+	level = abs( level );
 	for ( i=0; i<=level; ++i ) fprintf( outptr, "    " );
 }
 
@@ -33,6 +40,7 @@ output_tab1( FILE *outptr, int level, char *tag )
 	fprintf( outptr, "%s", tag );
 }
 
+#if 0
 static void
 output_tab2( FILE *outptr, int level, char *tag, char *data, int cr )
 {
@@ -40,6 +48,18 @@ output_tab2( FILE *outptr, int level, char *tag, char *data, int cr )
 	fprintf( outptr, "<%s>%s</%s>", tag, data, tag );
 	if ( cr ) fprintf( outptr, "\n" );
 }
+#endif
+
+static void
+output_tab2_attrib( FILE *outptr, int level, char *tag, char *data, 
+	char *attrib, char *type, int cr )
+{
+	output_tab0( outptr, level );
+	fprintf( outptr, "<%s", tag );
+	if ( attrib && type ) fprintf( outptr, " %s=\"%s\"", attrib, type );
+	fprintf( outptr, ">%s</%s>", data, tag );
+	if ( cr ) fprintf( outptr, "\n" );
+}
 
 static void
 output_tab4( FILE *outptr, int level, char *tag, char *aname, char *avalue,
@@ -54,11 +74,25 @@ static void
 output_fill2( FILE *outptr, int level, char *tag, fields *info, int n, int cr )
 {
 	if ( n!=-1 ) {
-		output_tab2( outptr, level, tag, info->data[n].data, cr );
+		output_tab2_attrib( outptr, level, tag, info->data[n].data, 
+			NULL, NULL, cr );
 		fields_setused( info, n );
 	}
 }
 
+#if 0
+static void
+output_fill2_attrib( FILE *outptr, int level, char *tag, char *attrib, char *type,
+	fields *info, int n, int cr )
+{
+	if ( n!=-1 ) {
+		output_tab2_attrib( outptr, level, tag, info->data[n].data, 
+			attrib, type, cr );
+		fields_setused( info, n );
+	}
+}
+#endif
+
 static void
 output_fill4( FILE *outptr, int level, char *tag, char *aname, char *avalue,
 		fields *info, int n, int cr )
@@ -78,10 +112,10 @@ output_title( fields *info, FILE *outptr, int level )
 	int shrttl = fields_find( info, "SHORTTITLE", level );
 
 	output_tab1( outptr, level, "<titleInfo>\n" );
-	output_fill2( outptr, level+1, "title", info, ttl, 1);
-	output_fill2( outptr, level+1, "subTitle", info, subttl, 1 );
+	output_fill2( outptr, increment_level(level,1), "title", info, ttl, 1);
+	output_fill2( outptr, increment_level(level,1), "subTitle", info, subttl, 1 );
 	if ( ttl==-1 && subttl==-1 ) 
-		output_tab1( outptr, level+1, "<title/>\n" );
+		output_tab1( outptr, increment_level(level,1), "<title/>\n" );
 	output_tab1( outptr, level, "</titleInfo>\n" );
 
 	/* output shorttitle if it's different from normal title */
@@ -126,7 +160,7 @@ output_name( FILE *outptr, char *p, int level )
 				part.data[1]='\0';
 			}
 			if ( n==0 ) output_personalstart( outptr, level );
-			output_tab4( outptr, level+1, "namePart", "type", 
+			output_tab4( outptr, increment_level(level,1), "namePart", "type", 
 					"given", part.data, 1 );
 			n++;
 		}
@@ -138,7 +172,7 @@ output_name( FILE *outptr, char *p, int level )
 
 	if ( family.len ) {
 		if ( n==0 ) output_personalstart( outptr, level );
-		output_tab4( outptr, level+1, "namePart", "type", "family",
+		output_tab4( outptr, increment_level(level,1), "namePart", "type", "family",
 				family.data, 1 );
 	}
 
@@ -149,104 +183,72 @@ output_name( FILE *outptr, char *p, int level )
 
 #define NO_AUTHORITY (0)
 #define MARC_AUTHORITY (1)
-#define NAME_ASIS (2)
-#define NAME_CORP (4)
 
 static void
 output_names( fields *info, FILE *outptr, int level )
 {
 	convert   names[] = {
 	  { "author",        "AUTHOR",             MARC_AUTHORITY },
-	  { "author",        "AUTHOR:ASIS",        MARC_AUTHORITY | NAME_ASIS },
-	  { "author",        "AUTHOR:CORP",        MARC_AUTHORITY | NAME_CORP },
 	  { "writer",        "WRITER",             MARC_AUTHORITY },
-	  { "writer",        "WRITER:ASIS",        MARC_AUTHORITY | NAME_ASIS },
-	  { "writer",        "WRITER:CORP",        MARC_AUTHORITY | NAME_CORP },
-	  { "patent holder", "ASSIGNEE",           MARC_AUTHORITY },
-	  { "patent holder", "ASSIGNEE:ASIS",      MARC_AUTHORITY | NAME_ASIS },
-	  { "patent holder", "ASSIGNEE:CORP",      MARC_AUTHORITY | NAME_CORP },
-	  { "editor",        "EDITOR",             MARC_AUTHORITY },
-	  { "editor",        "EDITOR:ASIS",        MARC_AUTHORITY | NAME_ASIS },
-	  { "editor",        "EDITOR:CORP",        MARC_AUTHORITY | NAME_CORP },
 	  { "artist",        "ARTIST",             MARC_AUTHORITY },
-	  { "artist",        "ARTIST:ASIS",        MARC_AUTHORITY | NAME_ASIS },
-	  { "artist",        "ARTIST:CORP",        MARC_AUTHORITY | NAME_CORP },
 	  { "cartographer",  "CARTOGRAPHER",       MARC_AUTHORITY },
-	  { "cartographer",  "CARTOGRAPHER:ASIS",  MARC_AUTHORITY | NAME_ASIS},
-	  { "cartographer",  "CARTOGRAPHER:CORP",  MARC_AUTHORITY | NAME_CORP},
+	  { "commentator",   "COMMENTATOR",        NO_AUTHORITY   },
 	  { "degree grantor","DEGREEGRANTOR",      MARC_AUTHORITY },
-	  { "degree grantor","DEGREEGRANTOR:ASIS", MARC_AUTHORITY | NAME_ASIS },
-	  { "degree grantor","DEGREEGRANTOR:CORP", MARC_AUTHORITY | NAME_CORP },
+	  { "director",      "DIRECTOR",           MARC_AUTHORITY },
+	  { "editor",        "EDITOR",             MARC_AUTHORITY },
 	  { "inventor",      "INVENTOR",           MARC_AUTHORITY },
-	  { "inventor",      "INVENTOR:ASIS",      MARC_AUTHORITY | NAME_ASIS},
-	  { "inventor",      "INVENTOR:CORP",      MARC_AUTHORITY | NAME_CORP},
 	  { "organizer of meeting","ORGANIZER",    MARC_AUTHORITY },
-	  { "organizer of meeting","ORGANIZER:ASIS",MARC_AUTHORITY| NAME_ASIS },
-	  { "organizer of meeting","ORGANIZER:CORP",MARC_AUTHORITY| NAME_CORP },
-	  { "director",      "DIRECTOR",           MARC_AUTHORITY },
-	  { "director",      "DIRECTOR:ASIS",      MARC_AUTHORITY | NAME_ASIS },
-	  { "director",      "DIRECTOR:CORP",      MARC_AUTHORITY | NAME_CORP },
+	  { "patent holder", "ASSIGNEE",           MARC_AUTHORITY },
 	  { "performer",     "PERFORMER",          MARC_AUTHORITY },
-	  { "performer",     "PERFORMER:ASIS",     MARC_AUTHORITY | NAME_ASIS },
-	  { "performer",     "PERFORMER:CORP",     MARC_AUTHORITY | NAME_CORP },
-	  { "reporter",      "REPORTER",           NO_AUTHORITY   },
-	  { "reporter",      "REPORTER:ASIS",      NO_AUTHORITY   | NAME_ASIS },
-	  { "reporter",      "REPORTER:CORP",      NO_AUTHORITY   | NAME_CORP },
-	  { "translator",    "TRANSLATOR",         MARC_AUTHORITY },
-	  { "director",      "DIRECTOR",           MARC_AUTHORITY },
-	  { "director",      "DIRECTOR:ASIS",      MARC_AUTHORITY | NAME_ASIS },
-	  { "director",      "DIRECTOR:CORP",      MARC_AUTHORITY | NAME_CORP },
-	  { "performer",     "PERFORMER",          MARC_AUTHORITY },
-	  { "performer",     "PERFORMER:ASIS",     MARC_AUTHORITY | NAME_ASIS },
-	  { "performer",     "PERFORMER:CORP",     MARC_AUTHORITY | NAME_CORP },
-	  { "translator",    "TRANSLATOR",         MARC_AUTHORITY },
-	  { "translator",    "TRANSLATOR:ASIS",    MARC_AUTHORITY | NAME_ASIS },
-	  { "translator",    "TRANSLATOR:CORP",    MARC_AUTHORITY | NAME_CORP },
 	  { "recipient",     "RECIPIENT",          MARC_AUTHORITY },
-	  { "recipient",     "RECIPIENT:ASIS",     MARC_AUTHORITY | NAME_ASIS },
-	  { "recipient",     "RECIPIENT:CORP",     MARC_AUTHORITY | NAME_CORP },
+	  { "redactor",      "REDACTOR",           NO_AUTHORITY   },
+	  { "reporter",      "REPORTER",           NO_AUTHORITY   },
+	  { "translator",    "TRANSLATOR",         MARC_AUTHORITY },
 	  { "author",        "2ND_AUTHOR",         MARC_AUTHORITY },
-	  { "author",        "2ND_AUTHOR:ASIS",    MARC_AUTHORITY | NAME_ASIS },
-	  { "author",        "2ND_AUTHOR:CORP",    MARC_AUTHORITY | NAME_CORP },
 	  { "author",        "3RD_AUTHOR",         MARC_AUTHORITY },
-	  { "author",        "3RD_AUTHOR:ASIS",    MARC_AUTHORITY | NAME_ASIS },
-	  { "author",        "3RD_AUTHOR:CORP",    MARC_AUTHORITY | NAME_CORP },
 	  { "author",        "SUB_AUTHOR",         MARC_AUTHORITY },
-	  { "author",        "SUB_AUTHOR:ASIS",    MARC_AUTHORITY | NAME_ASIS },
-	  { "author",        "COMMITTEE",          MARC_AUTHORITY | NAME_CORP },
-	  { "author",        "COURT",              MARC_AUTHORITY | NAME_CORP },
-	  { "author",        "LEGISLATIVEBODY",    MARC_AUTHORITY | NAME_CORP }
+	  { "author",        "COMMITTEE",          MARC_AUTHORITY },
+	  { "author",        "COURT",              MARC_AUTHORITY },
+	  { "author",        "LEGISLATIVEBODY",    MARC_AUTHORITY }
 	};
 	int       i, n, ntypes = sizeof( names ) / sizeof( convert );
+	newstr role;
+	int f_asis, f_corp;
 
+	newstr_init( &role );
 	for ( n=0; n<ntypes; ++n ) {
 		for ( i=0; i<info->nfields; ++i ) {
 			if ( info->level[i]!=level ) continue;
-			if ( strcasecmp(info->tag[i].data,names[n].internal) )
+			f_asis = f_corp = 0;
+			newstr_strcpy( &role, info->tag[i].data );
+			if ( newstr_findreplace( &role, ":ASIS", "" )) f_asis=1;
+			if ( newstr_findreplace( &role, ":CORP", "" )) f_corp=1;
+			if ( strcasecmp( role.data, names[n].internal ) )
 				continue;
-			if ( names[n].code & NAME_ASIS ) {
+			if ( f_asis ) {
 				output_tab0( outptr, level );
 				fprintf( outptr, "<name>\n" );
-				output_fill2( outptr, level+1, "namePart", info, i, 1 );
-			} else if ( names[n].code & NAME_CORP ) {
+				output_fill2( outptr, increment_level(level,1), "namePart", info, i, 1 );
+			} else if ( f_corp ) {
 				output_tab0( outptr, level );
 				fprintf( outptr, "<name type=\"corporate\">\n" );
-				output_fill2( outptr, level+1, "namePart", info, i, 1 );
+				output_fill2( outptr, increment_level(level,1), "namePart", info, i, 1 );
 			} else {
 				output_name(outptr, info->data[i].data, level);
 			}
-			output_tab1( outptr, level+1, "<role>\n" );
-			output_tab1( outptr, level+2, "<roleTerm" );
+			output_tab1( outptr, increment_level(level,1), "<role>\n" );
+			output_tab1( outptr, increment_level(level,2), "<roleTerm" );
 			if ( names[n].code & MARC_AUTHORITY )
 				fprintf( outptr, " authority=\"marcrelator\"");
 			fprintf( outptr, " type=\"text\">");
 			fprintf( outptr, "%s", names[n].mods );
 			fprintf( outptr, "</roleTerm>\n");
-			output_tab1( outptr, level+1, "</role>\n" );
+			output_tab1( outptr, increment_level(level,1), "</role>\n" );
 			output_tab1( outptr, level, "</name>\n" );
 			fields_setused( info, i );
 		}
 	}
+	newstr_free( &role );
 }
 
 static int
@@ -280,7 +282,7 @@ static void
 output_dateissued( fields *info, FILE *outptr, int level, int pos[3] )
 {
 	int nprinted = 0, i;
-	output_tab1( outptr, level+1, "<dateIssued>" );
+	output_tab1( outptr, increment_level(level,1), "<dateIssued>" );
 	for ( i=0; i<3 && pos[i]!=-1; ++i ) {
 		if ( nprinted>0 ) fprintf( outptr, "-" );
 		if ( i>0 && info->data[pos[i]].len==1 )
@@ -314,19 +316,19 @@ output_origin( fields *info, FILE *outptr, int level )
 	if ( found==-1 && datefound==-1 ) return;
 
 	output_tab1( outptr, level, "<originInfo>\n" );
-	output_fill2( outptr, level+1, "issuance", info, pos[0], 1 );
+	output_fill2( outptr, increment_level(level,1), "issuance", info, pos[0], 1 );
 	if ( datefound!=-1 ) output_dateissued( info, outptr, level, date );
 
 	for ( n=1; n<ntypes; n++ ) {
 		if ( pos[n]==-1 ) continue;
-		output_tab0( outptr, level+1 );
+		output_tab0( outptr, increment_level(level,1) );
 		fprintf( outptr, "<%s", origin[n].mods );
 		fprintf( outptr, ">" );
 		if ( origin[n].code ) {
 			fprintf( outptr, "\n" );
-			output_fill4( outptr, level+2, "placeTerm", "type",
-					"text", info, pos[n], 1 );
-			output_tab0( outptr, level+1 );
+			output_fill4( outptr, increment_level(level,2), 
+				"placeTerm", "type", "text", info, pos[n], 1 );
+			output_tab0( outptr, increment_level(level,1) );
 		} else {
 			fprintf( outptr, "%s", info->data[pos[n]].data );
 			fields_setused( info, pos[n] );
@@ -382,7 +384,7 @@ output_partdate( fields *info, FILE *outptr, int level, int *wrote_header )
 		*wrote_header = 1;
 	}
 
-	output_tab1( outptr, level+1, "<date>" );
+	output_tab1( outptr, increment_level(level,1), "<date>" );
 
 	if ( parts[0].code!=-1 ) {
 		fprintf( outptr, "%s", info->data[ parts[0].code ].data);
@@ -417,7 +419,7 @@ mods_output_detail( fields *info, FILE *outptr, int item, char *item_name,
 		int level )
 {
 	if ( item==-1 ) return;
-	output_tab0( outptr, level+1 );
+	output_tab0( outptr, increment_level(level,1) );
 	fprintf( outptr, "<detail type=\"%s\"><number>", item_name );
 	fprintf( outptr, "%s</number></detail>\n", info->data[item].data );
 	fields_setused( info, item );
@@ -435,12 +437,12 @@ static void
 mods_output_extents( fields *info, FILE *outptr, int start, int end,
 		int total, char *extype, int level )
 {
-	output_tab0( outptr, level+1 );
+	output_tab0( outptr, increment_level(level,1) );
 	fprintf( outptr, "<extent unit=\"%s\">\n", extype);
-	output_fill2( outptr, level+2, "start", info, start, 1 );
-	output_fill2( outptr, level+2, "end", info, end, 1 );
-	output_fill2( outptr, level+2, "total", info, total, 1 );
-	output_tab1 ( outptr, level+1, "</extent>\n" );
+	output_fill2( outptr, increment_level(level,2), "start", info, start, 1 );
+	output_fill2( outptr, increment_level(level,2), "end", info, end, 1 );
+	output_fill2( outptr, increment_level(level,2), "total", info, total, 1 );
+	output_tab1 ( outptr, increment_level(level,1), "</extent>\n" );
 }
 
 static void
@@ -523,40 +525,12 @@ output_part( fields *info, FILE *outptr, int level )
 static void
 output_genre( fields *info, FILE *outptr, int level )
 {
-	char *marc[] = { "abstract or summary", "art original", 
-		"art reproduction", "atlas", "autobiography", 
-		"bibliography", "book", "catalog", "chart", "comic strip",
-		"conference publication", "database", "dictionary",
-		"diorama", "directory", "discography", "drama",
-		"encyclopedia", "essay", "festschrift", "fiction",
-       		"filmography", "filmstrip", "flash card", "folktale",
-		"font", "game", "government publication", "graphic",
-		"globe", "handbook", "history", "humor, satire",
-		"index", "instruction", "interview", "kit",
-		"language instruction", "law report or digest",
-		"legal article", "legal case and case notes", "legislation",
-		"letter", "loose-leaf", "map", "memoir", "microscope slide",
-		"model", "motion picture", "multivolume monograph",
-		"newspaper", "novel", "numeric data", 
-		"online system or service", "patent", "periodical",
-		"picture", "poetry", "programmed text", "realia",
-		"rehersal", "remote sensing image", "reporting",
-		"review", "series", "short story", "slide", "sound",
-		"speech", "statistics", "survey of literature",
-		"technical drawing", "technical report", "theses",
-		"toy", "transparency", "treaty", "videorecording",
-		"web site" 
-	};
-	int nmarc = sizeof( marc ) / sizeof( char* );
-	int i, j, ismarc;
+	int i, ismarc;
 	for ( i=0; i<info->nfields; ++i ) {
 		if ( info->level[i]!=level ) continue;
 		if ( strcasecmp( info->tag[i].data, "GENRE" ) ) continue;
-		ismarc = 0;
-		for ( j=0; j<nmarc && ismarc==0; ++j ) {
-			if ( !strcasecmp( info->data[i].data, marc[j] ) )
-				ismarc = 1;
-		}
+		if ( marc_findgenre( info->data[i].data )!=-1 ) ismarc = 1;
+		else ismarc = 0;
 		output_tab1( outptr, level, "<genre" );
 		if ( ismarc ) 
 			fprintf( outptr, " authority=\"marcgt\"" );
@@ -568,28 +542,18 @@ output_genre( fields *info, FILE *outptr, int level )
 static void
 output_typeresource( fields *info, FILE *outptr, int level )
 {
-	char *marc[] = { "text", "cartographic", "notated music",
-		"sound recording", "still image", "moving image",
-		"three dimensional object", "software, multimedia",
-		"mixed material"
-	};
-	int nmarc = sizeof( marc ) / sizeof( char* );
-	int resource, j, ismarc;
-	resource = fields_find( info, "RESOURCE", level );
-	if ( resource!=-1 ) {
-		ismarc = 0;
-		for ( j=0; j<nmarc && ismarc==0; ++j ) 
-			if ( !strcasecmp( info->data[resource].data, marc[j] ))
-				ismarc = 1;
-		if ( ismarc==0 ) {
+	int n, ismarc = 0;
+	n = fields_find( info, "RESOURCE", level );
+	if ( n!=-1 ) {
+		if ( marc_findresource( info->data[n].data )!=-1 ) ismarc = 1;
+		if ( !ismarc ) {
 			fprintf( stderr, "Illegal typeofResource = '%s'\n",
-					info->data[resource].data );
+					info->data[n].data );
 		} else {
-			for ( j=0; j<=level; ++j ) fprintf( outptr, "    " );
-			fprintf( outptr, "<typeOfResource>%s</typeOfResource>\n",
-			      info->data[resource].data );
+			output_fill2( outptr, level, "typeOfResource", info, 
+				n, 1 );
 		}
-		fields_setused( info, resource );
+		fields_setused( info, n );
 	}
 }
 
@@ -647,7 +611,7 @@ output_key( fields *info, FILE *outptr, int level )
 		if ( info->level[i]!=level ) continue;
 		if ( !strcasecmp( info->tag[i].data, "KEYWORD" ) ) {
 			output_tab1( outptr, level, "<subject>\n" );
-			output_fill2( outptr, level+1, "topic", info, i, 1 );
+			output_fill2( outptr, increment_level(level,1), "topic", info, i, 1 );
 			output_tab1( outptr, level, "</subject>\n" );
 		}
 	}
@@ -692,19 +656,22 @@ output_url( fields *info, FILE *outptr, int level )
 {
 	int location = fields_find( info, "LOCATION", level );
 	int url = fields_find( info, "URL", level );
+	int pdflink = fields_find( info, "PDFLINK", level );
 	int i;
-	if ( url==-1 && location==-1 ) return;
+	if ( url==-1 && location==-1 && pdflink==-1 ) return;
 	output_tab1( outptr, level, "<location>\n" );
 	for ( i=0; i<info->nfields; ++i ) {
 		if ( info->level[i]!=level ) continue;
 		if ( !strcasecmp( info->tag[i].data, "URL" ) ) {
-			output_fill2( outptr, level+1, "url", info, i, 1 );
+			output_fill2( outptr, increment_level(level,1), "url", info, i, 1 );
+		} else if ( !strcasecmp( info->tag[i].data, "PDFLINK" ) ) {
+			output_fill2( outptr, increment_level(level,1), "url",
+				/*"urlType", "pdf",*/ info, i, 1 );
 		}
 	}
-/*	if ( url!=-1 )
-		output_fill2( outptr, level+1, "url", info, url, 1 );*/
 	if ( location!=-1 )
-		output_fill2( outptr, level+1, "physicalLocation", info, location, 1 );
+		output_fill2( outptr, increment_level(level,1), "physicalLocation", info, 
+				location, 1 );
 	output_tab1( outptr, level, "</location>\n" );
 }
 
@@ -742,23 +709,43 @@ output_head( fields *info, FILE *outptr, int dropkey, unsigned long numrefs )
 	fprintf( outptr, ">\n" );
 }
 
+static int
+original_items( fields *info, int level )
+{
+	int i, targetlevel;
+	if ( level < 0 ) return 0;
+	targetlevel = -( level + 2 );
+	for ( i=0; i<info->nfields; ++i )
+		if ( info->level[i]==targetlevel ) return targetlevel;
+	return 0;
+}
+
 static void
 output_citeparts( fields *info, FILE *outptr, int level, int max )
 {
-	int i;
+	int orig_level;
 	output_title( info, outptr, level );
 	output_names( info, outptr, level);
 	output_origin( info, outptr, level );
 	output_type( info, outptr, level );
 	output_language( info, outptr, level );
 	output_description( info, outptr, level );
-	if ( level < max ) {
-		for ( i=0; i<=level; ++i ) fprintf( outptr, "    " );
-		fprintf( outptr, "<relatedItem type=\"host\">\n");
-		output_citeparts(info, outptr, level+1, max);
-		for ( i=0; i<=level; ++i ) fprintf( outptr, "    " );
+	if ( level>=0 && level < max ) {
+		output_tab0( outptr, level );
+		fprintf( outptr, "<relatedItem type=\"host\">\n" );
+		output_citeparts(info, outptr, increment_level(level,1), max );
+		output_tab0( outptr, level );
 		fprintf( outptr, "</relatedItem>\n");
 	}
+	/* Look for original item things */
+	orig_level = original_items( info, level );
+	if ( orig_level ) {
+		output_tab0( outptr, level );
+		fprintf( outptr, "<relatedItem type=\"original\">\n" );
+		output_citeparts( info, outptr, orig_level, max );
+		output_tab0( outptr, level );
+		fprintf( outptr, "</relatedItem>\n" );
+	}
 	output_abs( info, outptr, level );
 	output_timescited( info, outptr, level );
 	output_indexkey( info, outptr, level );
@@ -771,45 +758,44 @@ output_citeparts( fields *info, FILE *outptr, int level, int max )
 	output_part( info, outptr, level );
 }
 
-void
-modsout_write( fields *info, FILE *outptr, int format_opts, 
-			unsigned long numrefs )
+static void
+modsout_report_unused_tag( fields *info, int i, param *p, unsigned long numrefs )
 {
-	int i, max, dropkey;
+	if ( p->progname ) fprintf( stderr, "%s: ", p->progname );
+	fprintf( stderr, "Ref %lu unused tag: '%s' value: '%s' level: %d\n",
+			numrefs+1, info->tag[i].data, info->data[i].data,
+			info->level[i] );
+}
+
+static void
+modsout_report_unused_tags( fields *info, param *p, unsigned long numrefs )
+{
+	int i;
+	for ( i=0; i<info->nfields; ++i ) {
+		if ( info->used[i] ) continue;
+		modsout_report_unused_tag( info, i, p, numrefs );
+	}
+}
+
+void
+modsout_write( fields *info, FILE *outptr, param *p, unsigned long numrefs )
+{
+	int max, dropkey;
 	max = fields_maxlevel( info );
-	dropkey = ( format_opts & MODSOUT_DROPKEY );
+	dropkey = ( p->format_opts & MODSOUT_DROPKEY );
 
 	output_head( info, outptr, dropkey, numrefs );
 	output_citeparts( info, outptr, 0, max );
-	for ( i=0; i<info->nfields; ++i ) {
-		if ( !info->used[i] ) {
-			fprintf( stderr, "%s warning: ref %ld "
-				"unused tag: '%s' "
-				"value: '%s' level: %d\n", 
-				progname,
-				numrefs+1,
-				info->tag[i].data, 
-				info->data[i].data, info->level[i] );
-		}
-	}
+	modsout_report_unused_tags( info, p, numrefs );
+
 	fprintf( outptr, "</mods>\n" );
 	fflush( outptr );
 }
 
-static void
-modsout_writebom( FILE *outptr )
-{
-	int i, nc;
-	unsigned char code[6];
-	nc = utf8_encode( 0xFEFF, code );
-	for ( i=0; i<nc; ++i )
-		fprintf(outptr,"%c",code[i]);
-}
-
 void
-modsout_writeheader( FILE *outptr, int format )
+modsout_writeheader( FILE *outptr, param *p )
 {
-	if ( format & MODSOUT_BOM ) modsout_writebom( outptr );
+	if ( p->utf8bom ) utf8_writebom( outptr );
 	fprintf(outptr,"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
 	fprintf(outptr,"<modsCollection xmlns=\"http://www.loc.gov/mods/v3\">\n");
 }
