@@ -11,16 +11,15 @@
 
 /* format-specific options */
 #define WORDOUT_DROPKEY (2)
-#define WORDOUT_BOM     (4)
 
 #include <stdio.h>
-#include <stdlib.h>
 #include "bibl.h"
+#include "bibutils.h"
 
-extern void wordout_writeheader( FILE *outptr, int unicode );
+extern void wordout_writeheader( FILE *outptr, param *p );
 extern void wordout_writefooter( FILE *outptr );
-extern void wordout_write( fields *info, FILE *outptr,
-	int format_opts, unsigned long numrefs );
+extern void wordout_write( fields *info, FILE *outptr, param *p,
+	unsigned long numrefs );
 
 #endif
 
